"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chain_1 = require("@rill/chain");
var error_1 = require("@rill/error");
var http_1 = require("@rill/http");
var https_1 = require("@rill/https");
var extend = require("just-extend");
var path_to_regexp_1 = require("path-to-regexp");
var T = require("./_types");
exports.Types = T;
var attach_1 = require("./attach");
var context_1 = require("./context");
var respond_1 = require("./respond");
var Rill = /** @class */ (function () {
    /**
     * @description
     * Creates a universal app that will run middleware for a incoming request.
     *
     * @example
     * const app = Rill()
     */
    function Rill() {
        /** The current middleware stack. */
        this.stack = [];
        /* istanbul ignore next */
        if (!(this instanceof Rill)) {
            return new Rill();
        }
    }
    /**
     * @description
     * Takes the current middleware stack, chains it together and
     * returns a valid handler for a node js style server request.
     *
     * @example
     * const app = Rill()
     * app.use(...)
     * require('http').createServer(app.handler()).listen()
     */
    Rill.prototype.handler = function () {
        var fn = chain_1.default(this.stack);
        /**
         * Handles a node js server request and pushes it through a rill server.
         *
         * @param {http.IncomingMessage} req - The http request.
         * @param {http.ServerResponse} res - The http response.
         * @return {void}
         */
        return function (req, res) {
            res.statusCode = 404;
            var ctx = new context_1.Context(req, res);
            fn(ctx)
                .catch(function (err) {
                if (Number(ctx.res.status) === 404) {
                    ctx.res.status = 500;
                }
                /* istanbul ignore next */
                if (!(err instanceof error_1.default)) {
                    // tslint:disable-next-line
                    console && console.error && console.error(err);
                }
            })
                .then(function () { return respond_1.respond(ctx); });
        };
    };
    /**
     * @description
     * Creates a node server from the Rill server.
     *
     * @example
     * app.createServer().listen(3000)
     *
     * @param tls Optional HTTPS/TLS options (key, cert, etc).
     */
    Rill.prototype.createServer = function (tls) {
        var handler = this.handler();
        var server = tls
            ? https_1.createServer(tls, handler)
            : http_1.createServer(handler);
        // Setup link hijacking in the browser.
        /* istanbul ignore next */
        attach_1.default(server);
        return server;
    };
    /**
     * @description
     * Creates a node server from the current Rill server and starts listening for http requests.
     *
     * @example
     * rill().use(...).listen({ port: 3000 })
     *
     * @param options An object with configuration for binding the server to a port and ip.
     * @param onListening A function that is called once the server has started.
     */
    Rill.prototype.listen = function (options, onListening) {
        // Make options optional.
        if (typeof options === "function") {
            onListening = options;
            options = undefined;
        }
        options = (options || {});
        options.port = options.port != null ? options.port : 0;
        return this.createServer(options.tls).listen(options.port, options.ip, options.backlog, onListening);
    };
    /**
     * @description
     * Simple syntactic sugar for functions that
     * wish to modify the current rill instance.
     *
     * @example
     * app.setup(self => {
     *  // Modify the current app.
     *  self.use(...)
     *  self.modified = true
     * })
     *
     * @param setups A list of functions to run during initialization.
     */
    Rill.prototype.setup = function () {
        var setups = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            setups[_i] = arguments[_i];
        }
        for (var _a = 0, setups_1 = setups; _a < setups_1.length; _a++) {
            var setup_1 = setups_1[_a];
            if (!setup_1) {
                continue;
            }
            else if (typeof setup_1 === "function") {
                setup_1(this);
            }
            else {
                throw new TypeError("Rill#setup: Setup must be a function or falsey.");
            }
        }
        return this;
    };
    /**
     * @description
     * Append new middleware to the current rill application stack.
     *
     * @example
     * rill.use(fn1, fn2)
     *
     * @param middlewares A list of middleware to add to the app.
     */
    Rill.prototype.use = function () {
        var middlewares = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            middlewares[_i] = arguments[_i];
        }
        this.stack = this.stack.concat(middlewares);
        return this;
    };
    /**
     * @description
     * Use middleware at a specific pathname.
     *
     * @example
     * rill.at("/test", (ctx, next) => ...)
     *
     * @param pathname The pathname to match before running the middlewares.
     * @param middlewares A list of middlewares to add to the app.
     */
    Rill.prototype.at = function (pathname) {
        var middlewares = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            middlewares[_i - 1] = arguments[_i];
        }
        if (typeof pathname !== "string") {
            throw new TypeError("Rill#at: Path name must be a string.");
        }
        var keys = [];
        var reg = toReg(pathname, keys, { end: false, delimiter: "/" });
        var fn = chain_1.default(middlewares);
        return this.use(function (ctx, next) {
            var matchPath = ctx.req.matchPath;
            var matches = matchPath.match(reg);
            // Check if we matched the whole path.
            if (!matches || matches[0] !== matchPath) {
                return next();
            }
            // Check if params match.
            for (var i = keys.length; i--;) {
                var key = keys[i];
                var match = matches[i + 1];
                if (key.repeat) {
                    match = match == null ? [] : match.split("/");
                }
                ctx.req.params[key.name] = match;
            }
            // Update path for nested routes.
            var matched = matches[matches.length - 1] || "";
            if (ctx.req.matchPath !== matched) {
                ctx.req.matchPath = "/" + matched;
            }
            // Run middleware.
            return fn(ctx, function () {
                // Reset nested pathname before calling later middleware.
                ctx.req.matchPath = matchPath;
                // Run sibling middleware.
                return next();
            });
        });
    };
    /**
     * @description
     * Use middleware at a specific hostname.
     *
     * @example
     * app.host("test.com", (ctx, next) => ...)
     *
     * @param hostname The hostname to match before running the middlewares.
     * @param middlewares A list of middlewares to add to the app.
     */
    Rill.prototype.host = function (hostname) {
        var middlewares = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            middlewares[_i - 1] = arguments[_i];
        }
        if (typeof hostname !== "string") {
            throw new TypeError("Rill#host: Host name must be a string.");
        }
        var keys = [];
        var reg = toReg(hostname, keys, { strict: true, delimiter: "." });
        var fn = chain_1.default(middlewares);
        return this.use(function (ctx, next) {
            var matchHost = ctx.req.matchHost;
            var matches = matchHost.match(reg);
            // Check if we matched the whole hostname.
            if (!matches || matches[0] !== matchHost) {
                return next();
            }
            // Here we check for the dynamically matched subdomains.
            for (var i = keys.length; i--;) {
                var key = keys[i];
                var match = matches[i + 1];
                if (key.repeat) {
                    match = match == null ? [] : match.split(".");
                }
                ctx.req.subdomains[key.name] = match;
            }
            // Update hostname for nested routes.
            var matched = matches[matches.length - 1] || "";
            if (ctx.req.matchHost !== matched) {
                ctx.req.matchHost = matched;
            }
            // Run middleware.
            return fn(ctx, function () {
                // Reset nested hostname.
                ctx.req.matchHost = matchHost;
                // Run sibling middleware.
                return next();
            });
        });
    };
    return Rill;
}());
exports.default = Rill;
// Attach all http verbs as shortcut methods.
http_1.METHODS.forEach(function (method) {
    var name = method.toLowerCase();
    /**
     * Use middleware on |method| requests at an (optional) pathname.
     *
     * @example
     * app.|method|('/test', ...)
     */
    Rill.prototype[name] = handleMethodMiddleware;
    function handleMethodMiddleware(pathname) {
        var middlewares = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            middlewares[_i - 1] = arguments[_i];
        }
        if (typeof pathname !== "string") {
            middlewares.unshift(pathname);
            pathname = undefined;
        }
        var fn = chain_1.default(middlewares);
        if (pathname) {
            return this.at(pathname, matchMethod);
        }
        else {
            return this.use(matchMethod);
        }
        function matchMethod(ctx, next) {
            if (ctx.req.method !== method) {
                return next();
            }
            return fn(ctx, next);
        }
    }
});
/**
 * Small wrapper around path to regexp that treats a splat param "/*" as optional.
 * This makes mounting easier since typically when you do a path like "/test/*" you also want to treat "/test" as valid.
 */
function toReg(pathname, keys, options) {
    // First parse path into tokens.
    var tokens = path_to_regexp_1.parse(pathname);
    // Find the last token (checking for splat params).
    var splat = tokens[tokens.length - 1];
    // Check if the last token is a splat and make it optional.
    if (splat && splat.asterisk) {
        splat.optional = true;
    }
    // Convert the tokens to a regexp.
    var re = path_to_regexp_1.tokensToRegExp(tokens, options);
    // Assign keys to from regexp.
    re.keys = keys;
    for (var i = 0, len = tokens.length; i < len; i++) {
        if (typeof tokens[i] === "object") {
            keys.push(tokens[i]);
        }
    }
    return re;
}
// Support commonjs and esmodules.
module.exports = extend(exports.default, exports);
//# sourceMappingURL=index.js.map