"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var byte_length_1 = require("byte-length");
var content_check_1 = require("content-check");
var is_typeof_1 = require("is-typeof");
var statuses = require("statuses");
/**
 * @description
 * Runs general clean up on a request context and ends it with proper headers and status codes.
 *
 * @param ctx The Rill Context to send a response for.
 * @internal
 */
function respond(_a) {
    var req = _a.req, res = _a.res;
    var body = res.body;
    var original = res.original;
    var isStream = is_typeof_1.isStream(body);
    var isBuffer = is_typeof_1.isBuffer(body);
    if (
    // Allow skipping response.
    res.respond === false ||
        // Skip request ended externally.
        original.headersSent) {
        return;
    }
    // Apply default statuses.
    if (Number(res.status) === 404) {
        if (res.get("Location")) {
            // Ensure redirect status.
            res.status = 302;
        }
        else if (body) {
            // Default the status to 200 if there is substance to the response.
            res.status = 200;
        }
    }
    // Default status message based on status code.
    res.message = res.message || statuses[res.status];
    // Ensure no content-type for empty responses.
    if (req.method === "HEAD" || statuses.empty[res.status] || !body) {
        body = null;
        res.remove("Content-Type");
        res.remove("Content-Length");
    }
    else {
        // Attempt to guess content type.
        if (!res.get("Content-Type")) {
            res.set("Content-Type", content_check_1.check(body));
        }
        // Stringify objects that are not buffers.
        if (typeof body === "object" && !isStream && !isBuffer) {
            body = JSON.stringify(body);
        }
        // Attempt to guess content-length.
        if (!res.get("Content-Length") && !isStream) {
            res.set("Content-Length", String(byte_length_1.byteLength(body)));
        }
    }
    // Send off headers.
    original.writeHead(res.status, res.message, removeEmptyHeaders(res.headers));
    if (res.end === false) {
        // Allow for requests to stay open.
        return;
    }
    // Finish response.
    if (isStream) {
        body.pipe(original);
    }
    else {
        original.end(body);
    }
}
exports.respond = respond;
/**
 * @description
 * Utility to remove empty values from an object.
 *
 * @example
 * removeEmptyHeaders({ a: [], b: null, c: 0 }) // -> { c: 0 }
 *
 * @param obj The http headers to send.
 * @internal
 */
function removeEmptyHeaders(obj) {
    for (var key in obj) {
        if (obj[key] == null || obj[key].length === 0) {
            delete obj[key];
        }
    }
    return obj;
}
//# sourceMappingURL=respond.js.map